#!/usr/bin/perl -w -I/usr/sausalito/perl -I. -I/usr/sausalito/handlers/base/maillist
# $Id: sync_subscribers.pl,v 1.2.2.4 2002/02/07 02:27:50 pbaltz Exp $
#
# Counts subscribers for mailing lists GUI
# System.MailList .site .commit

my $DEBUG = 0;
$DEBUG && warn `date`."$0\n";

use MailList; # should be a local file
use CCE;
my $cce = new CCE;
$cce->connectfd();

my $oid = $cce->event_oid();
my($ok, $obj) = $cce->get($oid, 'MailList');

my %crit;
$crit{'site'} = $obj->{site} if ($obj->{site});

# get the fqdn so, local subscribers get counted correctly
my $fqdn = '';
if ($obj->{site}) {
	my ($vs_oid) = $cce->find('Vsite', { 'name' => $obj->{site} });
	($ok, my $vsite) = $cce->get($vs_oid);
	if (!$ok) {
		$cce->bye('FAIL', '[[base-maillist.systemError]]');
		exit(1);
	}
	$fqdn = $vsite->{fqdn};
} else {
	my ($sys_oid) = $cce->find('System');
	($ok, my $sys) = $cce->get($sys_oid);
	if (!$ok) {
		$cce->bye('FAIL', '[[base-maillist.systemError]]');
		exit(1);
	}
	$fqdn = $sys->{hostname} . '.' . $sys->{domainname};
}

$DEBUG && warn "Site: $obj->{site}\n";

my (@lists) = $cce->find('MailList', { 'site' => $obj->{site} });
foreach my $oid (@lists) {
	$DEBUG && warn "Fetching oid: $oid\n";

	my($ok, $ml) = $cce->get($oid);
	next unless ($ok);

	my $list = '/usr/local/majordomo/sites/'.
		$ml->{site} . '/lists/' . $ml->{name};

	next unless (-r $list);
	open(LIST, $list) || next;

	my($local, $remote) = ('&', '&');

	while (<LIST>) {
		chomp;
		next if (/^\s*$/o);
		next if (/\&/o); # control character

		#
		# listings with this fqdn are local if the specified user
		# name is a site member.  Server administrators can end up
		# having an email with the same fqdn as the site this list
		# belongs too, but they should show up as remote subscribers.
		#
		my $is_local = 0;
		if (/^([^\@]+)\@$fqdn$/) {
			#
			# make sure the email is a site member before assuming
			# they are local
			#
			my ($uoid) = $cce->find('User',
						{
							'name' => $1,
							'site' => $ml->{site}
						});
			if ($uoid || ($1 eq 'nobody')) {
				# really is a local user
				$_ = $1;
				$is_local = 1;
			} else {
				# not a site member
				$_ = "$1\@$fqdn";
			}
		} else {
			# still might not be a user from this site
			my $user = $_;
			my ($uoid) = $cce->find('User',
						{
							'name' => $_,
							'site' => $ml->{site}
						});
			$_ = $user;
			if ($uoid || ($user eq 'nobody')) {
				# local user who should be local for this list
				$is_local = 1;
			}
		}
		if(!$is_local) {
			$remote .= $_ . '&';
		} else {
			$local .= $_ . '&';
		}
	}
	close(LIST);

	$local = '' if ($local eq '&');
	$remote = '' if ($remote eq '&');

	$DEBUG && warn "List oid: $oid\nLocal:\n$local\nRemote:\n$remote\n";

	my ($aok) = $cce->set($oid, '',
			      {
				'remote_recips'=>$remote,
				'local_recips'=>$local,
			      });
}

$cce->bye('SUCCESS');
exit(0);

